package com.yonge.cooleshow.biz.dal.execl.analysis;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.enums.CellExtraTypeEnum;
import com.alibaba.excel.metadata.Sheet;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.microsvc.toolkit.middleware.excel.analysis.AnalysisContext;
import com.microsvc.toolkit.middleware.excel.analysis.DataAnalysis;
import com.microsvc.toolkit.middleware.excel.util.DownloadManager;
import com.yonge.cooleshow.biz.dal.entity.TenantGroup;
import com.yonge.cooleshow.biz.dal.execl.entity.StudentImportWrapper;
import com.yonge.cooleshow.biz.dal.execl.listener.StudentImportListener;
import com.yonge.cooleshow.biz.dal.service.EasyExcelService;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


@Slf4j
@Getter
@Setter
public class StudentImportAnalysis extends DataAnalysis<StudentImportAnalysis> {


    private List<StudentImportWrapper.StudentImport> studentImportWrappers;

    // Excel解析服务
    private EasyExcelService easyExcelService;

    private final Map<String, StudentImportWrapper.StudentImport> recordMap = Maps.newHashMap();
    private final Map<String, Long> subjectMap = Maps.newHashMap();
    private final Map<String, Long> tenantMap = Maps.newHashMap();
    private final Map<String, List<TenantGroup>> tenantGroupMap = Maps.newHashMap();
    private final List<String> phones = Lists.newArrayList();
    private final List<String> existPhones = Lists.newArrayList();
    public StudentImportAnalysis(AnalysisContext context) {
        super(context);
    }

    public static StudentImportAnalysis build(EasyExcelService easyExcelService) {
        StudentImportAnalysis analysis = new StudentImportAnalysis(AnalysisContext.builder().build());
        analysis.setEasyExcelService(easyExcelService);
        analysis.setStudentImportWrappers(Lists.newArrayList());
        return analysis;

    }


    /**
     * 读取文件数据到内存
     *
     * @param path 本地文件地址
     */
    @Override
    public void read(String path) {

        File file = new File(path);
        List<Sheet> sheets = EasyExcel.read(file).build().getSheets();
        if (CollectionUtils.isNotEmpty(sheets)) {

            sheets.forEach(item -> {
                // 读取数据
                EasyExcel.read(file, StudentImportWrapper.StudentImport.class, new StudentImportListener(getStudentImportWrappers(), item.getSheetName()))
                        .extraRead(CellExtraTypeEnum.MERGE).sheet(item.getSheetName()).doRead();
            });

        }

        // 删除本地缓存文件
        DownloadManager.getInstance().deleteOnExit(path);
    }

    /**
     * 校验数据合法
     *
     * @return Map<Integer, String>
     */
    @Override
    public Map<Integer, String> verify() {

        // 加载校验数据参数
        if (Objects.nonNull(getStudentImportWrappers())) {
            loadVerifyParamData();
        }

        Map<Integer, String> verifyDataMap = Maps.newLinkedHashMap();

        // 校验每条入库记录数据合法性
        StringBuffer ret = new StringBuffer();
        for (StudentImportWrapper.StudentImport item : getStudentImportWrappers()) {

            if (StringUtils.isEmpty(item.getUserName())) {
                ret.append("学生姓名不能为空;");
            }else {
                if(!item.getUserName().matches("^[\\u4e00-\\u9fa5]{2,10}$")){
                    ret.append("真实姓名格式不正确;");
                }
            }
            if (StringUtils.isEmpty(item.getPhone())) {
                ret.append("手机号不能为空;");
            }else {
                //手机号校验
                if (!item.getPhone().matches("^1[3-9]\\d{9}$")) {
                    ret.append("手机号格式不正确;");
                }else {
                    //校验是否重复
                    if(phones.contains(item.getPhone())){
                        ret.append("手机号重复;");
                    }else {
                        if(existPhones.contains(item.getPhone())){
                            ret.append("手机号已存在;");
                        }
                        phones.add(item.getPhone());
                    }
                }
            }
            if (StringUtils.isEmpty(item.getGender())) {
                ret.append("性别不能为空;");
            }
            if (StringUtils.isEmpty(item.getBirthday())) {
                ret.append("出生日期不能为空;");
            }else {
                if (!item.getBirthday().matches("^\\d{4}-\\d{2}-\\d{2}$")) {
                    ret.append("出生日期格式不正确;");
                }
            }
            if (StringUtils.isEmpty(item.getSubjectName())) {
                ret.append("声部不能为空;");
            }else {
                String[] split = item.getSubjectName().replaceAll(" ", "").replaceAll("，", ",").split(",");
                if (split.length > 1) {
                    ret.append("声部最多选择1个;");
                }
                if (!subjectMap.containsKey(split[0])) {
                    ret.append("声部无效:").append(split[0]).append(";");
                }
                item.setSubjectId(subjectMap.get(split[0]));
            }
            if (StringUtils.isEmpty(item.getTenantName())) {
                ret.append("机构名称不能为空;");
            }else {
                if("平台".equals(item.getTenantName())){
                    item.setTenantId(-1L);
                }else {
                    if (!tenantMap.containsKey(item.getTenantName())) {
                        ret.append("机构无效:").append(item.getTenantName()).append(";");
                    }
                    item.setTenantId(tenantMap.get(item.getTenantName()));
                }
            }
            if (StringUtils.isEmpty(item.getTenantGroupName())) {
                if(!"平台".equals(item.getTenantName())){
                    ret.append("小组名称不能为空;");
                }
            }else {
                if (!tenantGroupMap.containsKey(item.getTenantGroupName())) {
                    ret.append("小组名称无效:").append(item.getTenantGroupName()).append(";");
                }
                List<TenantGroup> tenantGroups = tenantGroupMap.get(item.getTenantGroupName());
                //根据机构id和小组名称获取小组id
                if (CollectionUtils.isNotEmpty(tenantGroups)) {
                    TenantGroup tenantGroup = tenantGroups.stream().filter(t -> t.getTenantId().equals(item.getTenantId())).findFirst().orElse(null);
                    if (Objects.nonNull(tenantGroup)) {
                        item.setTenantGroupId(tenantGroup.getId());
                    } else {
                        ret.append("小组名称无效:").append(item.getTenantGroupName()).append(";");
                    }
                }
            }
            if (StringUtils.isNotEmpty(ret)) {
                verifyDataMap.put(item.getRowIndex(), ret.toString());
                ret = new StringBuffer(); // 重置数据状态
            }
        }

        // 数据校验记录信息
        getContext().total(getStudentImportWrappers().size()).failure(verifyDataMap.size())
                .setSuccess(getContext().getTotal() - getContext().getFailure());

        return verifyDataMap;
    }

    /**
     * 数据转换
     */
    @Override
    public void convert() {

    }

    /**
     * 加载校验数据参数
     */
    private void loadVerifyParamData() {
        //获取所有的手机号和身份证号
        List<String> phones = getStudentImportWrappers().stream()
                .map(StudentImportWrapper.StudentImport::getPhone)
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(phones)) {
            existPhones.addAll(easyExcelService.getPhones(phones,"STUDENT"));
        }
        tenantGroupMap.putAll(easyExcelService.getTenantGroupMap());
        subjectMap.putAll(easyExcelService.getSubjectMap());
        tenantMap.putAll(easyExcelService.getTenantMap());
    }


    /**
     * 数据对象
     * @return RoomInfoAnalysis
     */
    @Override
    public StudentImportAnalysis get() {
        return this;
    }

}
